#include <iostream>
#include <iomanip>
#include <fstream>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_roots.h>


using namespace std;

// these global vars are initialized from parameters file
// defaults set here are overridden by that file

int NUMT=8;
long int STEPS=4000,UPDATE=100,SNAPUPDATE=1000;
double B=0.0,AT=0.05,EPS=0.001;
double ETAOS=0.3;
double TSTART=0.5,TF=0.1,TINIT=1.0;
double IC=0;
int PTASIZE,PHIPASIZE;



//controls value of tau_Pi
double COEFF=3.0;

//flags
int wflag=0;
int reachedTf=0;
int bflag=0;

int dof=37;

//also for convenience
long int globali;
double globalx;
//defined in loadeos
long int length;

//radius of nucleus in fm
double Rnuc=6.4;
//wood-saxon parameter in fm;
double anuc=0.54;


//for the equation of state
double *eoT4,*cs2i,*poT4,*Ti;

// these hold the current values
double ***u,**e,**pixy,**pixx,**piyy;

double **cyminp;

double atuomas=Rnuc/125;
double g2mua=0.43328;
double ttuomas;

//overall time
double t = 0;

//center of lattice 
int Middle;




//for convenience 
//global definition of ut(sx,sy) -- in order not to have it calculated
//a gazillion times
double globut;

double thf[4];
double dtpixx[4],dtpixy[4],dtpiyy[4];
double mypixt,mypiyt,mypitt,mypiee;

// output files
fstream cym,inited;

//splines -- for fancy freeze-out
gsl_interp_accel *wac;
gsl_spline *workspline;
//splines -- for equation of state
gsl_interp_accel *pacc,*Tacc,*cs2acc;
gsl_spline *pspline,*Tspline,*cs2spline;

//to know where to stop interpolation
double lowestE;

// initialize global arrays
void allocateMemory() {

cout << "==> Allocating memory\n";


 u = new double**[2];

 for (int i=0;i<2;i++) 
   u[i] = new double*[NUMT+2];

 for (int i=0;i<2;i++)
   for (int j=0;j<NUMT+2;j++) 
     u[i][j] = new double[NUMT+2];

	 
 e = new double*[NUMT+2];

 for (int i=0;i<NUMT+2;i++) 
    e[i] = new double[NUMT+2];
 
 pixy = new double*[NUMT+2];

 for (int i=0;i<NUMT+2;i++) 
   pixy[i] = new double[NUMT+2];


 pixx = new double*[NUMT+2];

 for (int i=0;i<NUMT+2;i++) 
   pixx[i] = new double[NUMT+2];

 piyy = new double*[NUMT+2];

 for (int i=0;i<NUMT+2;i++) 
   piyy[i] = new double[NUMT+2];

 //allocating memory
  cyminp = new double*[500];
  for (int i=0;i<500;i++) 
    cyminp[i] = new double[500];


}


void loadeos()
{
  fstream eosf;

  //ideal equation of state
  //eosf.open("idEOS.dat", ios::in);

  //qcd equation of state from Mikko Laine and York Schroeder, 
  //hep-ph/0603048
  //eosf.open("qcdEOS.dat",ios::in);
  //interpolated equation of state -- numerically more stable
  eosf.open("qcdIEOS.dat",ios::in);

  if (eosf.is_open())
    {
      eosf >> length;
      cout << "Read length of " << length << endl;

      

      eoT4 = new double[length];
      cs2i = new double[length];
      poT4 = new double[length];
      Ti = new double[length];

      for (int i=1;i<=length;i++)
	{
	  eosf >> Ti[i-1];
	  eosf >> eoT4[i-1];
	  eosf >> poT4[i-1];
	  eosf >> cs2i[i-1];
	}

      eosf.close();
    }
  else
    cout << "Could not open EOS file" << endl; 

  //interpolate
  pacc=gsl_interp_accel_alloc (); 
  Tacc=gsl_interp_accel_alloc (); 
  cs2acc=gsl_interp_accel_alloc (); 
  pspline=gsl_spline_alloc (gsl_interp_cspline, length);
  Tspline=gsl_spline_alloc (gsl_interp_cspline, length);
  cs2spline=gsl_spline_alloc (gsl_interp_cspline, length);
  
  
  double warrx[length];
  double warry[length];

  for (int i=0;i<length;i++)
    {
      warrx[i]=eoT4[i]*Ti[i]*Ti[i]*Ti[i]*Ti[i];
      warry[i]=poT4[i]*Ti[i]*Ti[i]*Ti[i]*Ti[i];
    }

  lowestE=warry[0];

  gsl_spline_init (pspline,warrx,warry,length);
  
  for (int i=0;i<length;i++)
    {
      warry[i]=Ti[i];
    }  
  gsl_spline_init (Tspline,warrx,warry,length);
  
  for (int i=0;i<length;i++)
    {
      warry[i]=cs2i[i];
    }  
  gsl_spline_init (cs2spline,warrx,warry,length);

}

//Wood-Saxon routine, in physical units
double WS(double x, double y, double z)
{
  
  //cout <<"R= " << R << endl;

  double temp;
  temp=x*x+y*y+z*z;
  temp=sqrt(temp);
  temp-=Rnuc;
  temp/=anuc;
  return 1/(1+exp(temp));
}

//transverse density T_A, in physical units
//(x, y in fm)
double TA(double x, double y)
{
  double temp=0;
  for (int i=0;i<1200;i++)
    temp+=WS(x,y,(i+0.5)*Rnuc/400.)*Rnuc/400.;

  //return result normalized to a gold nucleus
  return 2*temp*197./1175.22;
}


//number density of wounded nucleons
//from Kolb et. al, hep-ph/0103234
double getwnuc(double xx,double yy,double b)
{
  
  double mTAp=TA(xx+b/2.,yy);
  double mTAm=TA(xx-b/2.,yy);

  //return mTA*2*(1.-pow(1-mTA/197.*4.,197.));
  double temp=0;
  temp+=mTAp*(1.-exp(-mTAm*4.));
  temp+=mTAm*(1.-exp(-mTAp*4.));
  return temp;
}


//number density of participating nucleons
//from Kolb et. al, hep-ph/0103234
double getpart(double xx,double yy,double b)
{
  
  double mTAp=TA(xx+b/2.,yy);
  double mTAm=TA(xx-b/2.,yy);

  //return mTA*2*(1.-pow(1-mTA/197.*4.,197.));
  double temp=0;
  
  temp=4*mTAp*mTAm;

  return temp;
}

double getscal(double ss)
{
  long int i;
  for (i=0;i<length;i++)
    {
      if ((Ti[i]*Ti[i]*Ti[i]*(eoT4[i]+poT4[i]))>ss)
	break;
    }

  double temp=eoT4[i];
  temp*=Ti[i]*AT;
  temp*=Ti[i]*AT;
  temp*=Ti[i]*AT;
  temp*=Ti[i]*AT;

  return temp;
}


double Nwounded(double b)
{
  int DIVIDER=10;
  double temp=0;
  for (double xx=-2*Rnuc;xx<2*Rnuc;xx+=Rnuc/DIVIDER)
    for (double yy=-2*Rnuc;yy<2*Rnuc;yy+=Rnuc/DIVIDER)
      {
	temp+=getwnuc(xx,yy,b);
      }
  return temp*Rnuc/DIVIDER*Rnuc/DIVIDER;
}

double Npart(double b)
{
  int DIVIDER=10;
  double temp=0;
  for (double xx=-2*Rnuc;xx<2*Rnuc;xx+=Rnuc/DIVIDER)
    for (double yy=-2*Rnuc;yy<2*Rnuc;yy+=Rnuc/DIVIDER)
      {
	temp+=getpart(xx,yy,b);
      }
  return temp*Rnuc/DIVIDER*Rnuc/DIVIDER;
}


double aniso()
{
  double diff=0,sum=0;
  
  for (int sx=1;sx<=NUMT;sx++)
    for (int sy=1;sy<=NUMT;sy++)
      {
	diff+=e[sx][sy]*((sy-Middle)*(sy-Middle)-(sx-Middle)*(sx-Middle));
	sum+=e[sx][sy]*((sx-Middle)*(sx-Middle)+(sy-Middle)*(sy-Middle));
      }

  return diff/sum;
}

//definition according to Bhalerao, nucl-th/0508009v3
double overlapS()
{
  double mx=0,my=0,mean=0;
  
  for (int sx=1;sx<=NUMT;sx++)
    for (int sy=1;sy<=NUMT;sy++)
      {
	mx+=e[sx][sy]*(sx-Middle)*(sx-Middle);
	my+=e[sx][sy]*(sy-Middle)*(sy-Middle);
	mean+=e[sx][sy];
      }

  return 2*M_PI*sqrt(mx*my)/mean;
}


//get Tuomas' results and smooth them
void prepareCYM(double bb)
{

  char buffer[50];


  switch ((int) round(10*TINIT/atuomas*g2mua))
    {
    case 10:
      {
	printf("Loading data for TINIT=1g2mu\n");
	sprintf(buffer,"endens00229.dat");
	ttuomas=2.315*atuomas;
	break;
      }
    case 20:
      {
	printf("Loading data for TINIT=2g2mu\n");
	sprintf(buffer,"endens00458.dat");
	ttuomas=4.605*atuomas;
	break;
      }
    case 30:
      {
	printf("Loading data for TINIT=3g2mu\n");
	sprintf(buffer,"endens00687.dat");
	ttuomas=6.895*atuomas;
	break;
      }
    case 40:
      {
	printf("Loading data for TINIT=4g2mu\n");
	sprintf(buffer,"endens00916.dat");
	ttuomas=9.185*atuomas;
	break;
      }
    case 50:
      {
	printf("Loading data for t=5g2mu TINIT=%f\n",TINIT/atuomas*g2mua);
	sprintf(buffer,"endens01145.dat");
	ttuomas=11.475*atuomas;
	break;
      }
    default:
      {
	printf("No available data for initial time %f\n",TINIT);
	exit(0);
      }
    }

  char regu[3];

  sprintf(regu,"05");

  char filenam[128];

  printf("Rnuc=%f\n",Rnuc);

  switch ((int) round(10*bb/Rnuc))
    {
    case 0: 
      {
	printf("Loading data for B=0*Rnuc\n");
	sprintf(filenam,"CYM/b00L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);
	break;
      }
    case 20: 
      {
	printf("Loading data for B=2*Rnuc\n");
	sprintf(filenam,"CYM/b02L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);
	break;
      }
    case 40: 
      {
	printf("Loading data for B=4*Rnuc\n");
	sprintf(filenam,"CYM/b04L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);
	break;
      }
    case 60: 
      {
	printf("Loading data for B=6*Rnuc\n");
	sprintf(filenam,"CYM/b06L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);
	break;
      } 
    case 80: 
      {
	printf("Loading data for B=8*Rnuc\n");
	sprintf(filenam,"CYM/b08L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);	
	break;
      }
    case 100: 
      {
	printf("Loading data for B=10*Rnuc\n");
	sprintf(filenam,"CYM/b10L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);	
	break;
      }
    case 120: 
      {
	printf("Loading data for B=12*Rnuc\n");
	sprintf(filenam,"CYM/b12L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);	
	break;
      }
    case 140: 
      {
	printf("Loading data for B=14*Rnuc\n");
	sprintf(filenam,"CYM/b14L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);	
	break;
      }
    case 160: 
      {
	printf("Loading data for B=16*Rnuc\n");
	sprintf(filenam,"CYM/b16L%s/%s",regu,buffer);
	cym.open(filenam, ios::in);	
	break;
      }
    default:
      {
	printf("No available data for impact parameter %f\n",B);
	exit(0);
      }
    }

  
  
  double temp;
  for (int i=0;i<500;i++)
    for (int j=0;j<500;j++)
      {
	cym >> temp;
	cyminp[i][j]=fabs(temp);
      }

  cym.close();

}

double smoothcym1(int sx,int sy)
{
  double temp=0;
  temp+=cyminp[sx][sy];
  temp+=cyminp[500-sx-1][sy];
  temp+=cyminp[sx][500-sy-1];
  temp+=cyminp[500-sx-1][500-sy-1];
  temp*=0.25;
  return temp;
}


double smoothcym2(int sx,int sy)
{
  double temp=0;
  int SMOOTHNESS=20;

  for (int i=-SMOOTHNESS;i<=SMOOTHNESS;i++)
    for (int j=-SMOOTHNESS;j<=SMOOTHNESS;j++)
      {
	if ((sx+i>=0)&&(sx+i<500)&&(sy+j>=0)&&(sy+j<500))
	  temp+=smoothcym1(sx+i,sy+j);
	else
	  {
	    printf("Error in smoothing: out of bounds\n");
	    printf("tried sx=%i sy=%i\n",sx,sy);
	    exit(0);
	  }
      }

  temp/=(2*SMOOTHNESS+1)*(2*SMOOTHNESS+1);

  return temp;
}

int trickindex(int s)
{
  if(s<=0)
    return (s+250);
  else
    return (s+249);
}




double getcym(double x,double y)
{
  int cx,cy;

  

  if (x>=0)
    cx=(int) (x/atuomas);
  else
    cx=(int) (x/atuomas-1);
  
  if (y>=0)
    cy=(int) (y/atuomas);
  else
    cy=(int) (y/atuomas-1);

  //using bilinear interpolation:
  //(Numerical recipes)

  double t=(x-cx*atuomas)/atuomas;
  
  double u=(y-cy*atuomas)/atuomas;
  
  int tcx,tcy;

  tcx=trickindex(cx);
  tcy=trickindex(cy);

  double y1,y2,y3,y4;

  y1=smoothcym2(tcx,tcy);
  y2=smoothcym2(tcx+1,tcy);
  y3=smoothcym2(tcx+1,tcy+1);
  y4=smoothcym2(tcx,tcy+1);

   double result;
  result=(1-t)*(1-u)*y1+t*(1-u)*y2+t*u*y3+(1-t)*u*y4;

  return result;
  
}

void cycleb()
{

  //Middle
  Middle=(NUMT-1)/2+1;
  //Middle=1;

  printf("mi %i\n",Middle);

  //freeze-out temperature
  TF=TF*AT;

  cout << "TF=" << TF/AT << endl;

  cout << "TSTART=" << TSTART << endl;

  //load equation of state

  loadeos();

//convert fm/c to lattice units
  t=TINIT*5.06842/AT;

  //locate starting temperature in EOS
  long int i;
  for (i=0;i<length;i++)
    {
      if (Ti[i]>TSTART)
	break;
    }
  
  cout << "found temperature" << Ti[i] << endl;
  //energy density at that temperature in physical units
  double e0;

  e0=(eoT4[i])*Ti[i]*AT;
  e0*=Ti[i]*AT;
  e0*=Ti[i]*AT;
  e0*=Ti[i]*AT;

  //printf("Impact=%f\t Nwound=%f\t Npart=%f \n",B,Nwounded(B),Npart(B));

  

  double normcym;
  normcym=AT/5.06842;
  normcym*=AT/5.06842;
  normcym*=AT/5.06842;
  normcym*=AT/5.06842;
  //setting g^2=4
  
  for (int k=0;k<8;k++)
    { 
      double b=(double) 2*k;

      prepareCYM(b);
      normcym/=4*atuomas*atuomas*atuomas*ttuomas;


      for (int sx=1;sx<=NUMT;sx++)
	for (int sy=1;sy<=NUMT;sy++)
	  {
	    //wounded nucleon scaling
	    //e[sx][sy]=e0*getwnuc((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842,b)/4.29048;
	    //participant scaling
	    //e[sx][sy]=e0*getpart((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842,b)/18.4151;
	    e[sx][sy]=normcym*getcym((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842);
	  }
  

      //printf("aniso %f\n",aniso());
      
      printf("{%f,%f},\t",b,aniso());
    }

  printf("\n");
  
  /*
    printf("Number of wounded nucs:\n");
  for (double bb=0;bb<15;bb+=1)
    {
      double temp=Nwounded(bb);
      //printf("%f\t %f\n",bb,temp);
      printf("{%f,%f},\t",bb,temp);
    }
  printf("\n");

  printf("Number of participating nucs:\n");
  for (double bb=0;bb<15;bb+=1)
    {
      double temp=Npart(bb);
      //printf("%f\t %f\n",bb,temp);
      printf("{%f,%f},\t",bb,temp);
    }
  printf("\n");
  */

  wac=gsl_interp_accel_alloc (); 
  workspline=gsl_spline_alloc (gsl_interp_cspline, Middle);
 
}


void setInitialConditions()
{

  double sig;
  //extern double randGauss(double); // generates a gaussian random number with mean 0

  //Middle
  Middle=(NUMT-1)/2+1;
  //Middle=1;

  printf("mi %i\n",Middle);

  //freeze-out temperature
  TF=TF*AT;

  cout << "TF=" << TF/AT << endl;

  cout << "TSTART=" << TSTART << endl;

  //load equation of state

  loadeos();

  //convert fm/c to lattice units
  t=TINIT*5.06842/AT;

  //locate starting temperature in EOS
  long int i;
  for (i=0;i<length;i++)
    {
      if (Ti[i]>TSTART)
	break;
    }
  
  cout << "found temperature" << Ti[i] << endl;
  //energy density at that temperature in physical units
  double e0;

  e0=(eoT4[i])*Ti[i]*AT;
  e0*=Ti[i]*AT;
  e0*=Ti[i]*AT;
  e0*=Ti[i]*AT;

  //entropy density at that temperature in physical units
  double s0;

  s0=(eoT4[i]+poT4[i]);
  s0*=Ti[i];
  s0*=Ti[i];
  s0*=Ti[i];
  
  //printf("Impact=%f\t Nwound=%f\t Npart=%f \n",B,Nwounded(B),Npart(B));

  double normcym;

  if (IC==-3)
    {
      prepareCYM(B);
      normcym=AT/5.06842;
      normcym*=AT/5.06842;
      normcym*=AT/5.06842;
      normcym*=AT/5.06842;
      //setting g^2=4
      normcym/=4*atuomas*atuomas*atuomas*ttuomas ;
    }

  if (IC<0)
    {
      if (IC==-1)
	printf("Glauber: ENpart scaling\n");
      if (IC==-2)
	printf("Glauber: ENcoll scaling\n");
      if (IC==-3)
	printf("CGC: Lappi's CYM\n");
    }
  else
    {
      printf("Glauber Entropy scaling: Ncoll admixture %f percent\n",IC*100);
    }

  double b=B;
  for (int sx=1;sx<=NUMT;sx++)
    for (int sy=1;sy<=NUMT;sy++)
      {
	if (IC<0)
	  {
	    //wounded nucleon scaling
	    if (IC==-1)
	      e[sx][sy]=e0*getwnuc((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842,b)/4.29048;
	    //participant scaling
	    if (IC==-2)
	      e[sx][sy]=e0*getpart((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842,b)/18.4151;
	    if (IC==-3)
	      e[sx][sy]=normcym*getcym((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842);
	  }
	else
	  {
	    e[sx][sy]=getscal((getwnuc((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842,b)/4.29048*(1-IC)+getpart((sx-Middle)*AT/5.06842,(sy-Middle)*AT/5.06842,b)/18.4151*IC)*s0);
	  }
	
      }
  
  cout << "ED at center " << e[Middle][Middle] << endl;

  cout << "overlap" << overlapS()*AT*AT << endl;


  wac=gsl_interp_accel_alloc (); 
  workspline=gsl_spline_alloc (gsl_interp_cspline, Middle);
 
}

void outputed()
{
  for (int sx=1;sx<=NUMT;sx++)
    for (int sy=1;sy<=NUMT;sy++)
      inited << e[sx][sy] <<"\t";

  inited<<endl;
}


int main() 
{
  
  extern void readParameters(char*);
  
  readParameters("data/params.txt");  
  
  allocateMemory();
 
  setInitialConditions();
  
  //cycleb();


  
  
  inited.open("data/inited.dat", ios::out);

  outputed();

  inited.close();
  
  printf("Done!\n");

  return 0;
}
